/******************************************************************************
 * cliFunc.c
 *
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdbool.h>

#include "cliArg.h"
#include "define.h"
#include "Hub.h"
#include "utility.h"
#include "hubFwUpdaterCLI.h"

/*
 * print command usage
 */
static void inline command_usage(char *prog, char *cmd, struct arg_handle *arg_handles, int arg_num)
{
    printf("%s %s %s\n", prog, cmd, (arg_num > 0 ? "[options]" : ""));
    if (arg_num > 0)
    {
        printf("List of options:\n");
        arg_print_usage(arg_handles, arg_num, stdout);
    }
    printf("\n");
}

/* help */
int help_func(int argc, char *argv[])
{
    cli_usage(argv[0]);

    return 0;
}

//
enum eIspParameters
{
    ISP_CMD_TYPE = 0,
    ISP_CMD_NUMBER,
    ISP_CMD_BIN,
};

enum eIspCmdType
{
    ISP_TYPE_NONE = 0,
    ISP_TYPE_INI,
    ISP_TYPE_SINGLE,
};

struct arg_handle isp_args[] = {
    {'t', "type", "ini or single", ARG_TYPE_STRING, {0}, 0, 0},
    {'n', "number", "hub number", ARG_TYPE_INT, {0}, 0, 1},
    {'b', "bin", "bin file name", ARG_TYPE_STRING, {0}, 0, 1},
};

int isp_arg_num = sizeof(isp_args) / sizeof(struct arg_handle);

int isp_firmware(int argc, char *argv[])
{
    int ret                   = false;
    int i                     = 0;
    int iIspCmdType           = 0;
    int iHubIndex             = 0;
    char type[MAX_BUF_LEN]    = {0};
    char binName[MAX_BUF_LEN] = {0};

    if ((ret = arg_get_arguments(isp_args, isp_arg_num, argc - 1, &(argv[1]))) != 0)
    {
        command_usage(argv[0], argv[1], isp_args, isp_arg_num);
        goto exit;
    }

    if (isp_args[ISP_CMD_TYPE].is_set == 0)
    {
        printf("You need to specify -%c [%s]\n", isp_args[ISP_CMD_TYPE].short_name, isp_args[ISP_CMD_TYPE].long_name);
        goto exit;
    }
    snprintf(type, sizeof(type), "%s", isp_args[ISP_CMD_TYPE].val.string);

    if (strcmp(type, "ini") == 0)
        iIspCmdType = ISP_TYPE_INI;
    else if (strcmp(type, "single") == 0)
        iIspCmdType = ISP_TYPE_SINGLE;
    else
        iIspCmdType = ISP_TYPE_NONE;

    if (iIspCmdType == ISP_TYPE_INI)
    {
        ret = HubIspInit();
        if (ret == false)
        {
            printf("HubIspInit fail\n");
            goto exit;
        }

        for (i = 0; i < m_devNum; i++)
        {
            ret = readBinFileFromIni(i);
            if (ret == false)
            {
                printf("Read binfile Error!\n");
                goto exit;
            }

            ret = DoIsp(i);
            if (ret == false)
            {
                printf("DoIsp fail hub no.%d\n", i + 1);
                goto exit;
            }
        }
    }
    else if (iIspCmdType == ISP_TYPE_SINGLE)
    {
        if (isp_args[ISP_CMD_NUMBER].is_set == 0)
        {
            printf("You need to specify -%c [%s]\n", isp_args[ISP_CMD_NUMBER].short_name,
                   isp_args[ISP_CMD_NUMBER].long_name);
            goto exit;
        }
        iHubIndex = isp_args[ISP_CMD_NUMBER].val.integer;

        if (isp_args[ISP_CMD_BIN].is_set == 0)
        {
            printf("You need to specify -%c [%s]\n", isp_args[ISP_CMD_BIN].short_name, isp_args[ISP_CMD_BIN].long_name);
            goto exit;
        }
        snprintf(binName, sizeof(type), "%s", isp_args[ISP_CMD_BIN].val.string);

        ret = HubIspInit();
        if (ret == false)
        {
            printf("HubIspInit fail\n");
            goto exit;
        }

        ret = readBinFileName(iHubIndex, binName);
        if (ret == false)
        {
            printf("Read binfile Error!\n");
            goto exit;
        }

        ret = DoIsp(iHubIndex);
        if (ret == false)
        {
            printf("DoIsp fail hub no.%d\n", i + 1);
            goto exit;
        }
    }
    else
    {
        printf("IspCmdType Error\n");
        goto exit;
    }

exit:
    CloseDevices();
    return 0;
}

int erase_firmware(int argc, char *argv[])
{
    int iHubIndex = 0;

    if (argc < 3)
    {
        printf("%s parameter is not enough, argc=%d\n", __func__, argc);
        goto exit;
    }

    iHubIndex = atoi(argv[2]);

    if (!hub_erase_firmware(iHubIndex))
        printf("hub_erase_firmware fail\n");

exit:
    return 0;
}

int read_fw_version(int argc, char *argv[])
{
    if (!hub_read_fw_version())
        printf("hub_read_fw_version fail\n");

    return 0;
}

int list_hub_tree(int argc, char *argv[])
{
    if (!hub_list_tree())
        printf("hub_list_tree fail\n");

    return 0;
}

int software_reset(int argc, char *argv[])
{
    int iHubIndex = 0;

    if (argc < 3)
    {
        printf("%s parameter is not enough, argc=%d\n", __func__, argc);
        goto exit;
    }

    iHubIndex = atoi(argv[2]);

    if (!hub_software_reset(iHubIndex))
        printf("hub_software_reset fail\n");

exit:
    return 0;
}

int switch_speed(int argc, char *argv[])
{
    int iHubIndex = 0;
    int iUsbSpeed = 0;

    if (argc < 4)
    {
        printf("%s parameter is not enough, argc=%d\n", __func__, argc);
        goto exit;
    }

    iHubIndex = atoi(argv[2]);
    iUsbSpeed = atoi(argv[3]);

    if (iUsbSpeed != 2 && iUsbSpeed != 3)
    {
        printf("%s speed parameter(%d) wrong, parameter must to be 2 or 3\n", __func__, iUsbSpeed);
        goto exit;
    }

    if (!hub_switch_speed(iHubIndex, iUsbSpeed))
        printf("hub_switch_speed fail\n");

exit:
    return 0;
}

int list_info(int argc, char *argv[])
{
    if (!hub_list_info())
        printf("hub_list_info fail\n");

    return 0;
}
