/******************************************************************************
 * iniParser.c
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdbool.h>
#include <unistd.h>
#include <ctype.h>

#include "iniParser.h"

// static function
static unsigned int strToHex(const char *s);

int GetCurrentPath(char buf[], const char *pFileName)
{
    char pidfile[64];
    int bytes = 0;
    char *p = 0;
    int fd = 0;
    size_t nFileNameLen = strlen(pFileName);

    sprintf(pidfile, "/proc/%d/cmdline", getpid());

    fd    = open(pidfile, O_RDONLY, 0);
    bytes = read(fd, buf, MAX_BUF_LEN);
    close(fd);

    if (bytes == 0)
    {
        printf("GetCurrentPath() bytes == 0\n");
        goto exit;
    }

    buf[bytes] = '\0';

    p = strrchr(buf, '/');
    if (p == 0)
    {
        p = buf;
    }
    else
    {
        p++;
    }

    // incase of memory buffer is not 0.
    memcpy(p, pFileName, nFileNameLen);
    *(p + nFileNameLen) = '\0';

exit:
    return 0;
}

char *GetIniKeyString(const char *title, const char *key, const char *filename)
{
    FILE *fp;
    char szLine[1024]        = {0};
    static char tmpstr[1024] = {0};
    int rtnval;
    int i     = 0;
    int flag  = 0;
    char *tmp = NULL;

    if ((fp = fopen(filename, "r")) == NULL)
    {
        printf("file not found!\n");
        return (char *)("");
    }
    while (!feof(fp))
    {
        rtnval = fgetc(fp);
        if (rtnval == EOF)
        {
            break;
        }
        else
        {
            szLine[i++] = rtnval;
        }
        if (rtnval == '\n')
        {
            szLine[--i] = '\0';
            i           = 0;
            tmp         = strchr(szLine, '=');

            if ((tmp != NULL) && (flag == 1))
            {
                // printf("key:%s, szLine:%s\n", key, szLine);
                if (strstr(szLine, key) != NULL)
                {
                    if ('#' == szLine[0])
                    {
                    }
                    else if (';' == szLine[0])
                    {
                        // do nothing.
                    }
                    else
                    {
                        memset(tmpstr, 0, sizeof(tmpstr));
                        strncpy(tmpstr, tmp + 1, strlen(tmp) - 2);
                        fclose(fp);
                        return tmpstr;
                    }
                }
            }
            else
            {
                strcpy(tmpstr, "[");
                strcat(tmpstr, title);
                strcat(tmpstr, "]");
                if (strncmp(tmpstr, szLine, strlen(tmpstr)) == 0)
                {
                    flag = 1;
                }
            }
        }
    }
    fclose(fp);
    return (char *)("");
}

int GetIniKeyInt(const char *title, const char *key, const char *filename)
{
    return atoi(GetIniKeyString(title, key, filename));
}

int GetIniKeyHex(const char *title, const char *key, const char *filename)
{
    return strToHex(GetIniKeyString(title, key, filename));
}

char *trimwhitespace(char *str)
{
    char *end;

    // Trim leading space
    while (isspace(*str))
        str++;

    if (*str == 0)  // All spaces?
        return str;

    // Trim trailing space
    end = str + strlen(str) - 1;
    while (end > str && isspace(*end))
        end--;

    // Write new null terminator
    *(end + 1) = 0;

    return str;
}

// Genesys flash list.
bool GetFlashInfo(const char *title,
                  const char *filename,
                  unsigned char flashinfo[FLASH_TYPE_NUMBER][FALSH_INFO_COUNT],
                  int *flashCount)
{
    bool ret = true;
    FILE *fp;
    char szLine[1024];
    static char tmpstr[1024] = {0};
    int rtnval;
    int i    = 0;
    int flag = 0;
    char *tmp;
    int flashCnt = 0;

    if ((fp = fopen(filename, "r")) == NULL)
    {
        printf("file not found!\n");
        ret = false;
        goto exit;
    }
    while (!feof(fp))
    {
        rtnval = fgetc(fp);
        if (rtnval == EOF)
        {
            break;
        }
        else
        {
            szLine[i++] = rtnval;
        }

        if (rtnval == '\n')
        {
            szLine[--i] = '\0';
            i           = 0;
            tmp         = strchr(szLine, '=');

            if ((tmp != NULL) && (flag == 1))
            {
                if ('[' == szLine[0])
                {
                    // end of FlashInfo section.
                    break;
                }
                else if ('#' == szLine[0])
                {
                    // do nothing.
                }
                else if (';' == szLine[0])
                {
                    // do nothing.
                }
                else
                {
                    int j       = 0;
                    char *delim = (char *)(" ");
                    char *pch;

                    memset(tmpstr, 0, sizeof(tmpstr));
                    strcpy(tmpstr, tmp + 1);

                    pch                    = strtok(tmpstr, delim);
                    flashinfo[flashCnt][j] = strToHex(pch);

                    j++;
                    for (j = 1; j < FALSH_INFO_COUNT; j++)
                    {
                        pch = strtok(NULL, delim);
                        if (pch)
                        {
                            flashinfo[flashCnt][j] = strToHex(pch);
                        }
                        else
                        {
                            flashinfo[flashCnt][j] = 0;
                        }
                    }
                    flashCnt++;
                }
            }
            else
            {
                strcpy(tmpstr, "[");
                strcat(tmpstr, title);
                strcat(tmpstr, "]");
                if (strncmp(tmpstr, szLine, strlen(tmpstr)) == 0)
                {
                    flag     = 1;
                    flashCnt = 0;
                }
            }
        }
    }

    *flashCount = flashCnt;

exit:
    fclose(fp);
    return ret;
}

static unsigned int strToHex(const char *s)
{
    unsigned int result = 0;
    int c;

    while (*s)
    {
        result = result << 4;
        if (c = (*s - '0'), (c >= 0 && c <= 9))
            result |= c;
        else if (c = (*s - 'A'), (c >= 0 && c <= 5))
            result |= (c + 10);
        else if (c = (*s - 'a'), (c >= 0 && c <= 5))
            result |= (c + 10);
        else
            break;
        ++s;
    }

    return result;
}

bool ReadFlashInfoFromIni(void)
{
    int flashCount = 0;
    bool status    = false;

    status = GetFlashInfo(FLASHINFO_SECTION, m_configFilePath, m_flashinfo, &flashCount);
    if (status == false)
    {
        printf("file not found!\n");
        status = false;
        goto exit;
    }
    m_flashcount = flashCount;

    status = true;
exit:
    return status;
}

void GetConfigFilePath(void)
{
    memset(m_configFilePath, 0, sizeof(m_configFilePath));
    GetCurrentPath(m_configFilePath, GLHUB_INI_FILE);
}

void ReadTargetFromIni()
{
    memset(m_szTargetVid, 0, 5);
    memcpy(m_szTargetVid, GetIniKeyString(GLHUB_SECTION, V_Vid, m_configFilePath), 4);
    
    if (m_szTargetVid[0] == '\0')
        memcpy(m_szTargetVid, GL_VID_STR, 4);

    memset(m_szTargetPid, 0, 5);
    memcpy(m_szTargetPid, GetIniKeyString(GLHUB_SECTION, V_Pid, m_configFilePath), 4);
    
    if (m_szTargetPid[0] == '\0')
        memcpy(m_szTargetPid, GL_PID_U2_STR, 4);
}
