#include <unistd.h>
#include "scan.h"
#include "Hub.h"

int send_control_cmd(
    int fdev, __u8 type, __u8 request, __u16 value, __u16 index, __u16 length, __u32 timeout, void *data, bool isvendor)
{
    int ret;
    struct usbdevfs_ctrltransfer vcmd;

    memset(&vcmd, 0, sizeof(struct usbdevfs_ctrltransfer));

    vcmd.bRequestType = type;
    vcmd.bRequest     = request;
    vcmd.wValue       = value;
    vcmd.wIndex       = index;
    vcmd.wLength      = length;
    vcmd.timeout      = timeout;
    vcmd.data         = data;

    if (isvendor)
    {
        ret = ioctl(fdev, USBDEVFS_CONTROL, &vcmd);
    }
    else
    {
        ret = ioctl(fdev, USBDEVFS_CONTROL, &vcmd);
    }

    // printf("ioctl of USBDEVFS_CONTROL return %d (> 0 for success)\n", ret);

    return ret;
}

int SendVendorCommand(int fdev, bool dirIn, __u8 bRequest, __u16 wValue, __u16 wIndex, int len, void *buf)
{
    int ret;

    ret = send_control_cmd(fdev, USB_RECIP_DEVICE | USB_TYPE_VENDOR | (dirIn ? USB_DIR_IN : USB_DIR_OUT), bRequest,
                           wValue, wIndex, len, CMD_TIMEOUT, buf, true);

    return ret;
}

int GetDescriptor_Device(int fdev, int len, void *buf)
{
    int ret;

    ret = send_control_cmd(fdev, USB_RECIP_DEVICE | USB_TYPE_STANDARD | USB_DIR_IN, USB_REQ_GET_DESCRIPTOR,
                           USB_DT_DEVICE << 8, 0, len, CMD_TIMEOUT, buf, false);

    return ret;
}

int GetDescriptor_String(int fdev, int index, int len, void *buf)
{
    int ret;

    ret = send_control_cmd(fdev, USB_RECIP_DEVICE | USB_TYPE_STANDARD | USB_DIR_IN, USB_REQ_GET_DESCRIPTOR,
                           USB_DT_STRING << 8 | index, 0, len, CMD_TIMEOUT, buf, false);

    return ret;
}

int check_vidpid(char *devf_path)
{
    int ret = false;
    __u8 buffer[32];
    FILE *fp = NULL;
    __u16 _vid, _pid;

    fp = fopen(devf_path, "rb+");
    if (fp == NULL)
    {
        printf("fopen devf_path:%s fail\n", devf_path);
        ret = false;
        goto exit;
    }

    fread(buffer, 32, 1, fp);

    _vid = *((__u16 *)(buffer + 8));
    _pid = *((__u16 *)(buffer + 10));

    // if(_vid != GL_VID || _pid != GL_PID_U2)
    if (_vid != GL_VID)
    {
        printf("_pid:%04x _vid:%04x not valid\n", _pid, _vid);
        ret = false;
        goto exit;
    }

    ret = true;
exit:
    if (fp)
    {
        fclose(fp);
    }
    return ret;
}
