#include <unistd.h>
#include <ctype.h>

#include "Hub.h"
#include "define.h"

int ASCIIConvert(unsigned char temp_value)
{
    char ch_temp = 0;
    int hex_val  = 0x00;

    ch_temp = tolower(temp_value);
    if ((ch_temp >= '0') && (ch_temp <= '9'))
        hex_val = ch_temp - '0';
    else if ((ch_temp >= 'a') && (ch_temp <= 'z'))
        hex_val = ch_temp - 'a' + 10;
    else if ((ch_temp >= 'A') && (ch_temp <= 'Z'))
        hex_val = ch_temp - 'A' + 10;

    return hex_val;
}

unsigned long calc_checksum(char* addr, ulong count)
{
    register unsigned long sum = 0;
    register unsigned char aa  = 0;

    while (count > 0)
    {
        aa = (*addr);
        sum += aa;
        addr++;
        count--;
    }

    return sum;
}

bool SortUsbTree(void)
{
    char buf[MAX_BUF_LEN]              = {0};
    char line[MAX_CHAR_PER_LINE]       = {0};
    bool status                        = false;
    FILE* fp                           = NULL;
    int i                              = 0;
    int hubNum                         = 0;
    int usb_list_bus                   = 0;
    int usb_list_dev                   = 0;
    GL_USB_HUB_NODE hub[GLHUB_MAX_NUM] = {0};

    m_devNum = 0;
    memset(m_szDev, 0, sizeof(m_szDev));

    // find GL hub buf & dev numbers
    sprintf(buf, "lsusb -d %s:%s > %s", m_szTargetVid, m_szTargetPid, LSUSB_FILE);  // modify this for filtering USB3 hub
    system(buf);

    if ((fp = fopen(LSUSB_FILE, "r")) == NULL)
    {
        status = false;
        printf("file %s not found!\n", LSUSB_FILE);
        goto exit;
    }

    while (fgets(line, MAX_CHAR_PER_LINE, fp))
    {
        char* ptr;
        int pos;

        memset(buf, 0, sizeof(buf));
        ptr = strstr(line, "Bus");
        pos = ptr - line;
        strncpy(buf, line + pos + 4, 3);
        hub[hubNum].busNum = atoi(buf);

        memset(buf, 0, sizeof(buf));
        ptr = strstr(line, "Device");
        pos = ptr - line;
        strncpy(buf, line + pos + 7, 3);
        hub[hubNum].devNum = atoi(buf);

        hubNum++;
        if (hubNum > GLHUB_MAX_NUM)
        {
            printf("reach GL hub max num %d\n", GLHUB_MAX_NUM);
            break;
        }
    }
    fclose(fp);
    fp = NULL;
    remove(LSUSB_FILE);

    memset(buf, 0, sizeof(buf));
    sprintf(buf, "lsusb -t > %s", LSUSBT_FILE);
    system(buf);
    if ((fp = fopen(LSUSBT_FILE, "r")) == NULL)
    {
        status = false;
        printf("file %s not found!\n", LSUSBT_FILE);
        goto exit;
    }

    while (fgets(line, MAX_CHAR_PER_LINE, fp))
    {
        char* ptr = NULL;
        int pos = 0, pos2 = 0;

        memset(buf, 0, sizeof(buf));
        ptr = strstr(line, "Bus");
        pos = ptr - line;

        if (ptr)
        {
            strncpy(buf, line + pos + 4, 2);
            usb_list_bus = atoi(buf);
        }
        else
        {
            if (usb_list_bus == 0)
            {
                continue;
            }
        }

        memset(buf, 0, sizeof(buf));
        ptr  = strstr(line, "Dev");
        pos  = ptr - line;
        ptr  = strstr(line + pos, ",");
        pos2 = ptr - line;
        strncpy(buf, line + pos + 4, pos2 - pos - 4);
        usb_list_dev = atoi(buf);

        for (i = 0; i < hubNum; i++)
        {
            if (hub[i].devNum == usb_list_dev && hub[i].busNum == usb_list_bus)
            {
                int fdev;
                int ret              = 0;
                char devf_path[2048] = {0};
                struct usb_device_descriptor dev_dsc;

                printf("GL hub %d found, bus:%d dev:%d \n", m_devNum, usb_list_bus, usb_list_dev);

                sprintf(devf_path, "%s/%03d/%03d", USBFS_ROOT, hub[i].busNum, hub[i].devNum);
                ret = check_vidpid(devf_path);
                if (ret == false)
                {
                    printf("check_vidpid fail. devf_path:%s\n", devf_path);
                    continue;
                }

                fdev = open(devf_path, O_RDWR);
                if (fdev == -1)
                {
                    printf("open devf_path fail.\n");
                    continue;
                }

                ret = GetDescriptor_Device(fdev, sizeof(dev_dsc), &dev_dsc);
                if (ret <= 0)
                {
                    printf("GetDescriptor_Device fail, ret:%d\n", ret);
                    continue;
                }
                strcpy(m_szDev[m_devNum], devf_path);
                m_devNum++;

                close(fdev);
            }
        }
    }
    fclose(fp);
    fp = NULL;
    remove(LSUSBT_FILE);

    status = true;
exit:
    if (fp != NULL)
    {
        fclose(fp);
    }
    return status;
}

bool WstringToString(unsigned char* pBuff, int iBuffSize, unsigned short* utf16, int iUtfLength)
{
    memset(pBuff, 0, iBuffSize);

    int iWriteByte = 0;

    for (int i = 0; i < iUtfLength; i++)
    {
        pBuff[iWriteByte] = utf16[i];

        iWriteByte++;
        if (iWriteByte == iBuffSize)
            break;
    }

    return true;
}
